@@ -1074,7 +1074,7 @@ void iscsi_rsp_set_residual(struct iscsi_cmd_rsp *rsp, struct scsi_cmd *scmd)
 struct iscsi_sense_data {
 	uint16_t length;
 	uint8_t  data[0];
-} __packed;
+} __attribute__((__packed__));
 
 static int iscsi_cmd_rsp_build(struct iscsi_task *task)
 {
