@@ -92,7 +92,7 @@ char *iser_portal_addr;
 struct iscsi_sense_data {
 	uint16_t length;
 	uint8_t data[0];
-} __packed;
+} __attribute__((__packed__));
 
 static size_t buf_pool_sz_mb = DEFAULT_POOL_SIZE_MB;
 static int cq_vector = -1;
