@@ -2,17 +2,17 @@
 #define __TARGET_H__
 
 #include <limits.h>
-#define BITS_PER_LONG (ULONG_MAX == 0xFFFFFFFFUL ? 32 : 64)
-#include "hash.h"
-
-#define	HASH_ORDER	4
-#define	hashfn(val)	hash_long((unsigned long) (val), HASH_ORDER)
 
 struct acl_entry {
 	char *address;
 	struct list_head aclent_list;
 };
 
+struct iqn_acl_entry {
+	char *name;
+	struct list_head iqn_aclent_list;
+};
+
 struct tgt_account {
 	int out_aid;
 	int nr_inaccount;
@@ -38,16 +38,18 @@ struct target {
 
 	struct list_head acl_list;
 
+	struct list_head iqn_acl_list;
+
 	struct tgt_account account;
 
-	struct bs_finish *bsf;
+	struct list_head lld_siblings;
 };
 
 struct it_nexus {
 	uint64_t itn_id;
 	long ctime;
 
-	struct list_head cmd_hash_list[1 << HASH_ORDER];
+	struct list_head cmd_list;
 
 	struct target *nexus_target;
 
@@ -57,7 +59,7 @@ struct it_nexus {
 	/* dirty hack for IBMVIO */
 	int host_no;
 
-	struct list_head it_nexus_lu_info_list;
+	struct list_head itn_itl_info_list;
 
 	/* only used for show operation */
 	char *info;
