@@ -55,13 +55,15 @@ static struct option const long_options[] = {
 	{"foreground", no_argument, 0, 'f'},
 	{"control-port", required_argument, 0, 'C'},
 	{"nr_iothreads", required_argument, 0, 't'},
+	{"pid-file", required_argument, 0, 'p'},
 	{"debug", required_argument, 0, 'd'},
+	{"nodaemonize", no_argument, 0, 'D'},
 	{"version", no_argument, 0, 'V'},
 	{"help", no_argument, 0, 'h'},
 	{0, 0, 0, 0},
 };
 
-static char *short_options = "fC:d:t:Vh";
+static char *short_options = "fDC:d:t:p:Vh";
 static char *spare_args;
 
 static void usage(int status)
@@ -75,8 +77,10 @@ static void usage(int status)
 	printf("Linux SCSI Target framework daemon, version %s\n\n"
 		"Usage: %s [OPTION]\n"
 		"-f, --foreground        make the program run in the foreground\n"
+		"-D, --nodaemonize       make the program run in the foreground with logger\n"
 		"-C, --control-port NNNN use port NNNN for the mgmt channel\n"
 		"-t, --nr_iothreads NNNN specify the number of I/O threads\n"
+		"-p, --pid-file filename specify the pid file\n"
 		"-d, --debug debuglevel  print debugging information\n"
 		"-V, --version           print version and exit\n"
 		"-h, --help              display this help and exit\n",
@@ -174,6 +178,28 @@ set_rlimit:
 	return 0;
 }
 
+void create_pid_file(const char *path)
+{
+	int fd, ret;
+	int mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
+	char buf[32] = {0};
+
+	fd = open(path, O_RDWR | O_CREAT, mode);
+	if (fd < 0) {
+		eprintf("can't create %s, %m\n", path);
+		return;
+	}
+
+	snprintf(buf, sizeof(buf), "%d", getpid());
+	ret = write(fd, buf, strlen(buf));
+	close(fd);
+
+	if (ret < 0) {
+		eprintf("can't write and remove %s, %m\n", path);
+		unlink(path);
+	}
+}
+
 int tgt_event_add(int fd, int events, event_handler_t handler, void *data)
 {
 	struct epoll_event ev;
@@ -523,8 +549,9 @@ int main(int argc, char **argv)
 	struct sigaction sa_old;
 	struct sigaction sa_new;
 	int err, ch, longindex, nr_lld = 0;
-	int is_daemon = 1, is_debug = 0;
+	int is_daemon = 1, is_debug = 0, use_logger = 1;
 	int ret;
+	char *pidfile = NULL;
 
 	sa_new.sa_handler = signal_catch;
 	sigemptyset(&sa_new.sa_mask);
@@ -543,6 +570,10 @@ int main(int argc, char **argv)
 				 &longindex)) >= 0) {
 		switch (ch) {
 		case 'f':
+			is_daemon = 0;
+			use_logger = 0;
+			break;
+		case 'D':
 			is_daemon = 0;
 			break;
 		case 'C':
@@ -555,6 +586,13 @@ int main(int argc, char **argv)
 			if (ret)
 				bad_optarg(ret, ch, optarg);
 			break;
+		case 'p':
+			pidfile = strdup(optarg);
+			if (pidfile == NULL) {
+				fprintf(stderr, "can't duplicate pid file\n");
+				exit(1);
+			}
+			break;
 		case 'd':
 			ret = str_to_int_range(optarg, is_debug, 0, 1);
 			if (ret)
@@ -593,7 +631,7 @@ int main(int argc, char **argv)
 	if (err)
 		exit(1);
 
-	err = log_init(program_name, LOG_SPACE_SIZE, is_daemon, is_debug);
+	err = log_init(program_name, LOG_SPACE_SIZE, use_logger, is_debug);
 	if (err)
 		exit(1);
 
@@ -621,6 +659,9 @@ int main(int argc, char **argv)
 	sd_notify(0, "READY=1\nSTATUS=Starting event loop...");
 #endif
 
+	if (is_daemon && pidfile)
+		create_pid_file(pidfile);
+
 	event_loop();
 
 	lld_exit();
@@ -631,5 +672,8 @@ int main(int argc, char **argv)
 
 	log_close();
 
+	if (is_daemon && pidfile)
+		unlink(pidfile);
+
 	return 0;
 }
